use `astboard`;

create table `sessions` (
  `session_id` binary(20) not null,
  `data` text,
  `modified` timestamp not null default '0000-00-00 00:00:00' on update current_timestamp,
  primary key (`session_id`)
) engine=InnoDB character set utf8;

create table `api_keys` (
  `api_key_id` int unsigned not null auto_increment,
  `key` varchar(255) not null,
  `description` varchar(255) default null,
  `enabled` enum('yes','no') default 'yes',
  `created` timestamp not null default '0000-00-00 00:00:00' on update current_timestamp,
  primary key (`api_key_id`),
  unique key `key` (`key`)
) engine=InnoDB character set utf8;

create table `modules` (
  `module_id` int unsigned not null auto_increment,
  `module` varchar(255) not null,
  `label` varchar(255) not null,
  primary key (`module_id`),
  unique key `name` (`module`)
) engine=InnoDB character set utf8;

create table `module_controllers` (
  `controller_id` int unsigned not null auto_increment,
  `module_id` int unsigned not null,
  `controller` varchar(255) not null,
  `label` varchar(255) default null,
  primary key (`controller_id`),
  unique key `controller` (`controller`),
  key `module_id` (`module_id`),
  foreign key (`module_id`) references `modules` (`module_id`) on delete cascade
) engine=InnoDB character set utf8;

create table `controller_actions` (
  `action_id` int unsigned not null auto_increment,
  `controller_id` int unsigned not null,
  `action` varchar(255) not null,
  `label` varchar(255) default null,
  primary key (`action_id`),
  key `controller_id` (`controller_id`),
  foreign key (`controller_id`) references `module_controllers` (`controller_id`) on delete cascade
) engine=InnoDB character set utf8;

create table `menu` (
  `menu_id` int unsigned not null auto_increment,
  `action_id` int unsigned default null,
  `label` varchar(255) default null,
  `link` varchar(255) default null,
  `parent` int unsigned not null default 0,
  `icon` varchar(255) null default null,
  `sort` int default 0,
  primary key (`menu_id`),
  key `action_id` (`action_id`),
  foreign key (`action_id`) references `controller_actions` (`action_id`) on delete cascade
) engine=InnoDB character set utf8;

create table `roles` (
  `role_id` int unsigned not null auto_increment,
  `name` varchar(255) not null,
  `default` enum('yes','no') not null default 'no',
  primary key (`role_id`)
) engine=InnoDB character set utf8;

create table `role_permissions` (
  `role_id` int unsigned not null,
  `action_id` int unsigned not null,
  foreign key (`action_id`) references `controller_actions` (`action_id`) on delete cascade,
  foreign key (`role_id`) references `roles` (`role_id`) on delete cascade
) engine=InnoDB character set utf8;

create table `users` (
  `user_id` int unsigned not null auto_increment,
  `role_id` int unsigned not null,
  `user` varchar(255) not null,
  `password` binary(64) default null,
  `active_account` enum('yes','no') default 'yes',
  `created_at` timestamp not null default '0000-00-00 00:00:00' on update current_timestamp,
  primary key (`user_id`),
  unique key `user` (`user`),
  foreign key (`role_id`) references `roles` (`role_id`) on delete cascade
) engine=InnoDB character set utf8;