var modal_element = null;

$(function(){
  configure_ajax();
  form_submit();
});

function configure_ajax(){
  $(document).ajaxStart(function() {
    $("#loader-container").show();
  }).ajaxStop(function() {
    $("#loader-container").hide();
  });
}

function form_submit(){
  $(document).on("submit", "form", function() {
    $(this).ajaxSubmit({
      type: "POST",
      success: process_server_response,
      url: window.location.pathname,
      resetForm: false,
      error: process_server_response
    });

    return false;
  });
}

function virtual_submit(data){
  $.ajax({
    type: "POST",
    url: window.location.pathname, //Only made request to the actual location
    data: data,
    success: function(Response){
      return process_server_response(Response);
    },
    error: process_server_error
  });
}

function core_request(method, data, callback){
  $.ajax({
    type: "POST",
    url: window.location.pathname, //Only made request to the actual location
    data: {coreMethod: method, data: data},
    success: function(Response){
      callback(Response);
    },
    error: process_server_error
  });
}

function server_request(action, data){
  $.ajax({
    type: "POST",
    url: window.location.pathname, //Only made request to the actual location
    data: {action: action, data: data},
    success: function(Response){
      return process_server_response(Response);
    },
    error: process_server_error
  });
}

function process_server_response(Response){
  close_open_modal();

  try{
    var data = JSON.parse(Response);
  }catch(err){
	  swal({
		  title: 'Unexpected Server Response',
		  type: "error",
		  text: Response
	  });
  }

  if(typeof data == 'object'){
    switch (data.action){
      case 'notification':
        notification(data);
        break;
      case 'append':
        append_content(data);
        loadModuleCallback(data);
        break;
      case 'replace':
        replace_content(data);
        loadModuleCallback(data);
        notification(data.notification);
        break;
      case 'update_content':
          update_content(data);
          break;
      case 'return':
        return data;
      case 'modal':
        show_modal(data);
	    loadModuleCallback(data);
        break;
      case 'reload':
        location.reload();
        break;
      case 'setup_value':
        setup_value(data);
        break;
      case 'confirmation':
        confirmation(data);
        break;
      case 'dependent_dropdown':
        dependent_dropdown(data);
        break;
      case 'validation_errors':
        validation_errors(data);
        break;
      case 'open_url':
        open_url(data);
        break;
      case 'module_menu':
        build_module_menu(data.menu);
        break;
      case 'gstack-item':
        build_gridstack_item(data);
        break;
      case 'exec_action':
        server_request(data.action_id);
        break;
      default:
        console.log(data);
        break;
    }

    delegate();
  }
}

function append_content(data){
  hide_modal_backdrop();
  $(data.args.place).append(data.content);
}

function replace_content(data){
  hide_modal_backdrop();
  $(data.args.place).html(data.content);
}

function update_content(data) {
	hide_modal_backdrop();
	$(data.args.id).remove();
	$(data.args.place).append(data.content);
}

function process_server_error(Response){
  console.log(Response.responseText);
}

function show_modal(data){
  var body = $("body");
  $("#" + data.dialog_id).remove();
  body.append(data.dialog);
  var dialog = modal_element = body.find("#" + data.dialog_id);
  dialog.modal('show');
}

function setup_value(data){
  var elem = $(document).find(data.element_id);
  elem.val(data.element_val);

    /*Special for licensing project*/
  var data_fg = elem.data('fg');
  $('.' + data_fg).removeClass('is-empty');
}

function close_open_modal(){
  if(modal_element !== null){
    modal_element.modal('hide');
    modal_element.remove();
    modal_element = null;
  }
}

function notification(data){
  if(typeof data.type === 'undefined'){
    return ;
  }

  toastr.options = {
    "closeButton": true,
    "newestOnTop": true,
    "positionClass": "toast-top-center",
    "closeDuration": 5000
  };

  switch(data.type){
    case 'info':
      toastr.info(data.message, data.title);
      break;

    case 'success':
      toastr.success(data.message, data.title);
      break;

    case 'warning':
      toastr.warning(data.message, data.title);
      break;

    case 'error':
      toastr.error(data.message, data.title);
      break;
  }
}

function confirmation(data){
  swal({
    title: data.title,
    text: data.msg,
    type: "warning",
    showCancelButton: true,
    confirmButtonColor: "#d15a65",
    confirmButtonText: data.confirm_button,
    closeOnConfirm: true
  },function(){
    server_request(data.action_id,{'confirmed': true,'id': data.identifier});
  });
}

function dependent_dropdown(data){
  var form = $(data.form);
  var child  = form.find(data.child);
  var options = data.options;

  child.html('');

  $.each(options, function (index, item) {
    var option = '';
    option += '<option value="'+ item.value +'"';
    if(typeof item.params !== 'undefined') {
      $.each(item.params, function (param_index, param_val){
        option += ' '+ param_index +'="'+ param_val +'"';
      });
    }
    option += '> ' + item.content + '</option>';

    child.append(option);
  });

  child.trigger('change');
}

function validation_errors(data){
  swal({
    title: data.title,
    type: "warning",
    text: data.content,
    html: true
  });
}

function load_resources_files(resources){
  if(typeof resources.js !== 'undefined'){
    $.each(resources.js, function(index, item){
      var js_source = 'resources/modules/' + item;
      $('<script />', {type: 'text/javascript', 'src': js_source}).appendTo('head');
    });
  }

  if(typeof  resources.css !== 'undefined'){
    $.each(resources.css, function(index, item){
      var css_resource =  'resources/modules/' + item;
      $('<link />', {href: css_resource, rel: 'stylesheet', type: 'text/css'}).appendTo('head')
    });
  }
}

function hide_modal_backdrop(){
  var b = $("body");
  b.removeClass('modal-open');
  b.find('div.modal-backdrop').remove();
}

function open_url(data){
  var win = window.open(window.location.origin + '/' + data.url, '_blank');
  win.focus();
}

function build_module_menu(menu_items) {
  var container = document.getElementById('sidebar-container');
  var ul = document.createElement('ul');

  $.each(menu_items, function(index, item){
    var li = document.createElement('li');
    li.setAttribute('title',item.title);

    var link = document.createElement('a');
    link.setAttribute('onclick', 'javascript:server_request('+ item.index_action +',' + item.identifier + ')');
    var logo = document.createElement('img');
    var src = (item.img != null && (item.img).length ? item.img : './resources/img/logo.png');
    logo.setAttribute('src', src);
    logo.setAttribute('class', 'img-circle');
    var content = document.createElement('div');
    content.setAttribute('class','clear');
    var title = document.createElement('span');
    title.setAttribute('class','title');
    title.innerHTML = item.title;
    var description = document.createElement('span');
    description.setAttribute('class','description');
    description.innerHTML = item.sub_title;

    content.appendChild(title);
    content.appendChild(description);
    link.appendChild(logo);
    link.appendChild(content);
    li.appendChild(link);
    ul.appendChild(li);
  });

  container.innerHTML = '';
  container.appendChild(ul);

  $("input#module-menu-filter").fastLiveFilter('#sidebar-container > ul', {
    timeout: 200
  });
}

function build_gridstack_item(data){
  hide_modal_backdrop();
  var widget = data.widget;
  var content = data.content;
  var gs = $(data.grid);
  var params = data.params;
  var grid = gs.data('gridstack');

 $.each(gs.find('.grid-stack-item'), function () {
	 if($(this).attr('id') === params.pwidget){
	   grid.removeWidget($(this));
     }
 });

  grid.addWidget($(content), params.posx, params.posy, params.width, params.height, params.autopos, params.min_width, null, params.min_height, null, widget);
}

function loadModuleCallback(data) {
    if(typeof data.call_back !== 'undefined'){
      launchModuleCallBack(data.call_back);
    }
}
