var socket_io;
var socket_reconnection_attempts = 0;
var js_data;
var speech_voices;
var default_voice;
var moduleCallBacks = {};
var environmentData = {};

$(document).ready(function() {
    init_form_validations();
    $("body").trigger("resize");
    populateVoicesList();
    fnBrowserDetect();
});

$(window).on('load resize', function () {
    if(typeof mobile_interaction === "function"){
        mobile_interaction();
    }

    configure_form_tabs_css();
});

function image_upload(form, element_name){
    var frm = $("#" + form);
    var file_input = frm.find("#" + element_name);
    var browse_btn = frm.find("#img-"+ element_name +"-browse");
    var change_btn = frm.find("#img-"+ element_name +"-change");
    var clear_btn =  frm.find("#img-prev-"+ element_name +"-clear");
    var default_img = frm.find("#img-"+ element_name +"-default");
    var user_img = frm.find("#img-"+ element_name +"-preview");

    file_input.on('change', function (){
        var file = this.files[0];
        var reader = new FileReader();

        if(typeof file != 'undefined'){
            reader.onload = function (e) {
                browse_btn.hide();
                change_btn.show();
                clear_btn.show();
                default_img.hide();
                user_img.attr('src', e.target.result);
                user_img.show();
            };

            reader.readAsDataURL(file);
        }
    });

    clear_btn.on('click', function (){
        file_input.val('');
        file_input.next().val('');
        clear_btn.hide();
        change_btn.hide();
        user_img.hide();
        browse_btn.show();
        default_img.show();
    });
}

function sanitize_number(number) {
    if(number !== undefined)
        return number.replace(/[^a-z0-9#*+]/gi,'');
    return 'unknown';
}

function create_modal(modal_title, callback) {
	var button = $("<button>").attr({"type" : "button", "data-dismiss" : "modal"});
	var modal = $("<div>").addClass("modal fade").attr("role", "dialog").append(
		$("<div>").addClass("modal-dialog").append(
			$("<div>").addClass("modal-content").append(
				$("<form>").addClass("form-horizontal").append(
					$("<div>").addClass("modal-header").append(
						button.clone().addClass("close").append($("<span>").html("&times;")),
						$("<h4>").addClass("modal-title").html(modal_title)
					),
					$("<div>").addClass("modal-body"),
					$("<div>").addClass("modal-footer").append(
						button.clone().addClass("btn btn-default").html("Cancel"),
						button.clone().addClass("btn btn-primary").html("Accept")
					)
				)
			)
		)
	);

	modal.on("hidden.bs.modal", function() { modal.remove(); });
	modal.on("click", ".btn-primary", function() { callback.apply(modal); });
	$("body").append(modal);
	return modal;
}

function populateVoicesList() {
	if(typeof speechSynthesis === 'undefined') {
		return ;
	}

	var timer = setInterval(function() {
		speech_voices = speechSynthesis.getVoices();
		if (speech_voices.length !== 0) {
			default_voice = speech_voices[0];
			clearInterval(timer);
		}
	}, 1000);
}

function app_update(btn) {
	btn.addClass('loading disabled');
	var old_btn_text = btn.text();
	var action = btn.attr('data-action');
	btn.text(btn.attr('data-updating-text'));

	$.ajax({
		type:'post',
		url: window.location.pathname,
		data: {action: action},
		success: function (response) {
			var data = JSON.parse(response);
			if(data.status === 'updated'){
				location.reload();
			}else{
				btn.text(old_btn_text);
				btn.removeClass('loading disabled');
				notification({
					type: 'error',
					message: data.message,
					title: data.title
				});
			}
		},
		error: process_server_error
	})
}

function addModuleCallBack(name, callback){
	moduleCallBacks[name] = callback;
}

function launchModuleCallBack(name) {
	if(name in moduleCallBacks){
		console.log('launching callback: ' + name);
		moduleCallBacks[name]();
	}else{
		console.log('Callback ' + name + ' does not exists');
	}
}

function iOS() {
	var iDevices = [
		'iPad Simulator',
		'iPhone Simulator',
		'iPod Simulator',
		'iPad',
		'iPhone',
		'iPod'
	];

	if (!!navigator.platform) {
		while (iDevices.length) {
			if (navigator.platform === iDevices.pop()){ return true; }
		}
	}

	return false;
}

function getContextualMenuTrigger() {
	var envSetting = Cookies.get('context_menu_trigger');
	if(typeof envSetting !== 'undefined'){
		return envSetting;
	}

	return 'left';
}

$.fn.serializeObject = function()
{
	var o = {};
	var a = this.serializeArray();
	$.each(a, function() {
		if (o[this.name]) {
			if (!o[this.name].push) {
				o[this.name] = [o[this.name]];
			}
			o[this.name].push(this.value || '');
		} else {
			o[this.name] = this.value || '';
		}
	});
	return o;
};

/**
 * It detects the current browser brand and add it as class to the application body
 */
function fnBrowserDetect(){
	let userAgent = navigator.userAgent;
	let browserName;

	if(userAgent.match(/chrome|chromium|crios/i)){
		browserName = "chrome";
	}else if(userAgent.match(/firefox|fxios/i)){
		browserName = "firefox";
	}  else if(userAgent.match(/safari/i)){
		browserName = "safari";
	}else if(userAgent.match(/opr\//i)){
		browserName = "opera";
	} else if(userAgent.match(/edg/i)){
		browserName = "edge";
	}else{
		browserName="unknown";
	}

	$("body").addClass(browserName)
	console.log("You are using "+ browserName +" browser");
}

function getMomentLocale() {
	var guiLocale = js_data.user_lang;
	guiLocale = guiLocale.toLowerCase();
	if(guiLocale === 'en_us')
		return 'en';

	var tempLocale = guiLocale.split("_");
	if(tempLocale[0] === tempLocale[1]){
		return tempLocale[0];
	}else{
		return tempLocale[0] + '-' + tempLocale[1];
	}
}