

$(function () {
    addModuleCallBack('core', function () {
	    var dialer = $('#dialer');
	    var dial_btn = $('#dial-btn');

	    dialer.easyAutocomplete({
		    data : js_data.extensions_auto_complete,
		    getValue: 'description',
		    list: {
			    match: {
				    enabled: true
			    },
			    onSelectItemEvent: function() {
				    var value = dialer.getSelectedItemData().extension;
				    dialer.val(value).trigger("change");
			    },
			    onKeyEnterEvent: function () {
				    var value = dialer.getSelectedItemData().extension;
				    dialer.val(value).trigger("change");
			    }
		    },
		    theme: "square",
		    adjustWidth: false
	    });

	    dial_btn.on('click', function () {
				sw.tws('call', {
					'extension': js_data.user_extension,
					'context': js_data.user_context,
					'callee': dialer.val()
				})

		    dialer.val('');
	    });

	    dialer.keypress(function (e) {
		    var key = e.which;
		    if(key === 13) {
			    dial_btn.click();
		    }
	    });

			astmonitor_connection(); //Init connection to node server

	    server_request('init_modules');
    })
});


function astmonitor_connection(){
	if(userAuth){
		console.log('Connecting to Live monitor...');
		loadAuthToken().then(connect_to_monitor, process_server_error);
	}
}

function connect_to_monitor(token){
	var protocol = (window.location.protocol === 'https:' ? 'wss' : 'ws');
	var port = window.location.port;
	var url = protocol + '://'+ window.location.hostname + '/';
	if(port.length && (port != 80 && port != 443))
		url = protocol + '://'+ window.location.hostname +':'+ port+'/';

	socket_io = io(url,{
		path: '/vpbx-monitor',
		forceNew: true,
		transports: ['websocket'],
		auth: {
			token: token
		}
	});

	socket_io.on('connect', function () {
		console.log("Live monitor is connected")
		socket_reconnection_attempts = 0;
		if(typeof document.getElementById("switchboard-content") !== "undefined"){
			//Refresh APP
			$("#sw-loader").hide();
			socket_io.emit('sw_opened');
		}
	});

	socket_io.on("connect_error", (err) => {
		if(typeof document.getElementById("switchboard-content") !== "undefined"){
			let loader = $("#sw-loader");

			//Clean board
			if(loader.is(":hidden")){
				callsClean();
				cleanConferences();
				parkingClean();
				cleanQueuedCalls();
				cleanQueueMembers();
				trunksClean();

				//Show loader
				loader.show();
			}
		}

		//Try to reconnect
		if(err?.message === 'unauthorized'){
			var timeout = (socket_reconnection_attempts === 0 ? 5000 : 60000)
			console.log("Trying to reconnect in the next " + (timeout / 1000) + ' seconds')
			setTimeout(function (){
				console.log('Trying to reconnect manually')
				socket_reconnection_attempts++;
				loadAuthToken().then(function (token) {
					socket_io.auth.token = token;
					socket_io.connect();
				})
			}, timeout)
		}
	});
}

function loadAuthToken(){
	return new Promise(function (resolve, reject) {
		$.ajax({
			dataType: 'json',
			type: "POST",
			timeout: 60000,
			url: window.location.pathname,
			data: {coreMethod: 'loadAuthToken'},
			success: function (data) {
				resolve(data.token);
			},
			error: reject
		});
	});
}