var extensions_summary_widget;

/**
 * Handle of widget callbacks
 */
$(function (){
	addModuleCallBack('extensions_summary_widget_settings', function () {
		launchModuleCallBack('layouts_build_widget');

		//Use the multi-select plugin
		var form = $("#m-form-l-widget-extensions_summary");
		var multiple_dropdown = form.find("select#items");
		multiple_dropdown.multiselect({'sortable': 'both', 'locale': js_data.user_lang});
	});

	addModuleCallBack('extensions_summary_widget', function () {
		extensions_summary_widget = $(".widget-extensions_summary");
		ExtensionSummaryInitiateLastCallsValues();

		//Catch Server Events
		socket_io.on('extension_summary', function (data){
			var extension = null;
			let type = data.type;
			type = type.charAt(0).toUpperCase() + type.slice(1);

			if(data.type !== 'local'){
				//The event return the extension number instead of the device name
				if(data.caller !== null && typeof extensions_devices_status[data.caller] !== 'undefined'){
					extension = data.caller;
					ExtensionSummaryUpdateCallCounter(extension, data.type, data.datetime);
				}

				//In the following cases, the device name is returned by the backend
				if(data.caller !== null && typeof devices[data.caller] !== 'undefined'){
					extension = devices[data.caller];
					ExtensionSummaryUpdateCallCounter(extension, data.type, data.datetime);
				}

				if(data.callee !== null && typeof devices[data.callee] !== 'undefined'){
					extension = devices[data.callee];
					ExtensionSummaryUpdateCallCounter(extension, data.type, data.datetime);
				}
			}
		});

		//Context Menu
		ExtensionsSummaryContextMenu();
	});
});

function ExtensionsSummaryContextMenu(){
	$.contextMenu({
		selector: '#switchboard-monitor .widget-extensions_summary i.with-queues',
		trigger: 'left',
		build: function(trigger, event) {
			var parent = trigger.attr("data-parent");
			var tdColumn = $(document).find("td#"+parent);

			var options = {
				callback: function (key, options) {
					queuesActions(key, options, tdColumn);
				},
				delay: 500,
				autoHide: true,
				items: {}
			};

			var data = {};

			$.ajax({
				type: "POST",
				dataType: "JSON",
				url: window.location.pathname, //Only made request to the actual location
				async: false,
				context: $(this),
				data: {
					action: $('div#switchboard-monitor').find('input[name="wm_action"]').val(),
					widget : tdColumn.attr('data-widget'),
					queues_menu: true,
					status : 'any',
					extension : tdColumn.attr('data-extension'),
					ext_id: tdColumn.attr('data-ext-id'),
					queues: tdColumn.attr('data-queues'),
					qal: ( tdColumn.hasClass('qal-active') ? 1 : 0),
					qap: ( tdColumn.hasClass('qap-active') ? 1 : 0)
				},
				success: function(response){
					options.items = response;
				},
				error: process_server_error
			});

			return options;
		}
	});
}

function ExtensionSummaryUpdateCallCounter(extension, type, datetime){
	var callCounter = extensions_summary_widget.find("td.ext-summary-" + extension + '.' + type +'_calls');
	var LastCallTime = extensions_summary_widget.find("td.ext-summary-" + extension + '.last_' + type +'_calls');
	var TotalCallCounter = extensions_summary_widget.find("td.ext-summary-" + extension + '.total_calls');
	var OutboundCallsCounter = extensions_summary_widget.find("td.ext-summary-" + extension + '.outbound_calls');
	var IncomingCallsCounter = extensions_summary_widget.find("td.ext-summary-" + extension + '.incoming_calls');
	var CurrentCalls = callCounter.text();

	var NewCalls = parseInt(CurrentCalls) + 1;

	callCounter.text(NewCalls);
	LastCallTime.html(moment.unix(datetime).fromNow());

	//Update TotalCounter
	TotalCallCounter.text((parseInt(OutboundCallsCounter.text()) + parseInt(IncomingCallsCounter.text())));
}

/**
 * Initiate the last call values using moment.js library
 */
function ExtensionSummaryInitiateLastCallsValues(){
    var classes = ['last_local_calls', 'last_outbound_calls', 'last_incoming_calls'];
    classes.forEach(function (c){
	    var tdColumn = extensions_summary_widget.find("td." + c);
	    tdColumn.each(function (i){
	    	var e = $(this);
			var last_call = e.text();

			if(last_call !== '0' && last_call !== '--'){
				e.html(moment.unix(last_call).fromNow());
			}else{
				e.html('--');
			}
	    });
    });
}

/**
 * Update the DND status on the extensions summary widget
 * @param device
 * @param enabled
 */
function ExtensionSummaryDND(device, enabled){
	if(typeof extensions_summary_widget !== "undefined"){
		var tdColumn = extensions_summary_widget.find("td.dnd-status[data-dnd=\""+ device +"\"]");
		if(tdColumn.length){
			if(enabled === true){
				tdColumn.addClass('active-status');
			}else{
				tdColumn.removeClass('active-status');
			}
		}
	}
}

/**
 * Update the Registration status on the extension summary widget
 * @param extension
 * @param registered
 */
function ExtensionSummaryUpdatePhoneStatus(extension, registered){
	if(typeof extensions_summary_widget !== "undefined"){
		var tdColumn = extensions_summary_widget.find("td.ext-summary-" + extension + ".phone-status");
		if(tdColumn.length){
			if(registered === true){
				tdColumn.addClass('active-status');
			}else{
				tdColumn.removeClass('active-status');
			}
		}
	}
}

function ExtensionSummarySetQueuesStatus(data) {
	extensions_summary_widget = $(".widget-extensions_summary");
	if(!extensions_summary_widget.length)
		return ;

	$.each(data, function (extension, queues) {
		var elem = extensions_summary_widget.find("td.ext-summary-" + extension + ".queue-status");

		if(!elem.length)
			return ;


		var pauseElem = extensions_summary_widget.find("td.ext-summary-" + extension + ".pausedreason");
		var InCallElem = extensions_summary_widget.find("td.ext-summary-" + extension + ".in_call");

		var obj = JSON.parse(JSON.stringify(queues));
		//Storage the value of the logged in status
		var logged_in = $.map(obj, function (data, key) {
			return data.logged_in;
		});

		//Storage the value of the paused status
		var paused = $.map(obj, function (data, key) {
			return data.paused;
		});

		//Storage the value of the paused reason
		var paused_reason = $.map(obj, function (data, key) {
			return data.paused_reason;
		});

		//Storage In Call Value
		var in_call = $.map(obj, function (data, key) {
			return data.in_call;
		});

		if($.inArray('yes', in_call) === -1){
			InCallElem.text(InCallElem.attr('data-no'));
		}else{
			InCallElem.text(InCallElem.attr('data-yes'));
		}

		//Set Paused Status
		if($.inArray('yes', paused) === -1){
			elem.removeClass('qap-active');
			pauseElem.text('---');
		}else{
			elem.addClass('qap-active');
			var pause = paused_reason.shift();
			pauseElem.text((pause.length ? pause : pauseElem.attr('data-text')));
		}

		//Set logged in status
		if($.inArray('yes', logged_in) === -1){
			elem.removeClass('qal-active');
			elem.removeClass('qap-active');
			pauseElem.text('---');
		}else{
			elem.addClass('qal-active');
		}
	});
}