var queued_calls_notifications = {};
var queued_call_notified = false;

$(function () {
	addModuleCallBack('queued_calls_widget', function () {
		socket_io.on('queue_caller_join', function (data) {
			var row = $(".queued-calls-items-" + data.queue);
			var ul = row.find('ul.list-group');
			var channel = data.channel;
			var li_class = channel.replace(/[\/.@;]/g,'_');
			var li = ul.find('li.' + li_class);
			if(!li.length){
				var new_li = $("<li>").attr({
					"class": "list-group-item " + li_class,
					"data-position": data.position,
					"data-channel": data.channel
				}).append(
					$("<span>").attr('class', 'pull-right timer').html(toHHMMSS(data.wait))

				).append(
					$("<span>").attr('class', 'label label-success position').html(data.position)
				).append(
					data.caller_number + " - " + data.caller_name
				);

				ul.append(new_li);

				var queued_calls_interval = setInterval(function () {
					var elem = ul.find('li.' + li_class);
					if(elem.length){
						var timer = elem.find('span.timer');
						var new_duration = parseInt(toSeconds(timer.text())) + 1;
						updateQueuedCallsColor(elem, new_duration);
						timer.text(toHHMMSS(new_duration));
					}else{
						clearInterval(queued_calls_interval);
					}
				}, 1000);
			}else{
				li.attr('data-position', data.position);
				li.find('span.timer').html(toHHMMSS(data.wait));
				li.find('span.position').html(data.position);
			}


			updateQueuedCallsColor(li, data.wait);
			sort_queued_calls(ul);
			queued_calls_context_menu(data.queue);
			if(js_data.queues_sound_notification){
				queued_call_notification(data.channel, data.queue);
			}
		});

		socket_io.on('queue_caller_leave', function (data) {
			var row = $(".queued-calls-items-" + data.queue);
			var ul = row.find('ul.list-group');
			var channel = data.channel;
			var li_class = channel.replace(/[\/.@;]/g,'_');
			var li = ul.find('li.' + li_class);
			if(li.length){
				li.contextMenu(false);
				li.remove();
			}

			queued_calls_notifications[channel] = undefined;
			sort_queued_calls(ul);
		});
	});

	addModuleCallBack('queued_calls_widget_settings', function () {
		launchModuleCallBack('layouts_build_widget');
	});
});

function updateQueuedCallsColor(elem, time){
	if(time > 20 && time < 30){
		elem.find('span.position').removeClass('label-success');
		elem.find('span.position').addClass('label-warning');
	}else if(time > 30){
		elem.find('span.position').removeClass('label-default');
		elem.find('span.position').removeClass('label-warning');
		elem.find('span.position').addClass('label-danger');
	}
}

function queued_calls_context_menu(queue) {
	$.contextMenu({
		selector: '#switchboard-monitor .queued-calls-items-' + queue + ' li',
		trigger: getContextualMenuTrigger(),
		build: function (trigger, event) {
			var options = {
				callback: function(key, options) {
					var channel = trigger.attr('data-channel');
					if(key === 'take_call'){
						if(typeof user_extension !== 'undefined' && user_extension.length){
							sw.tws('redirect_call', {
								'exten': user_extension,
								'context': user_context,
								'channel': channel
							})
						}else{
							toastr.warning('You have not an extension assigned', 'Extensions');
						}

					}else if(key === 'redirect_call'){
						virtual_submit({
							action: $('div#switchboard-monitor').find('input[name="wf_action"]').val(),
							widget : trigger.parent('ul').attr('data-widget'),
							channel: channel,
							form: 'transfer_queued_call'
						});
					}
				},
				delay: 500,
				autoHide: true,
				items: {}
			};
			$.ajax({
				type: "POST",
				dataType: "JSON",
				url: window.location.pathname, //Only made request to the actual location
				async: false,
				context: $(this),
				data: {
					action: $('div#switchboard-monitor').find('input[name="wm_action"]').val(),
					widget : trigger.parent('ul').attr('data-widget'),
					status : 'queued'
				},
				success: function(response){
					options.items = response;
				},
				error: process_server_error
			});

			return options;
		}
	});
}

function redirect_call(modal) {
	var selected_extension = modal.find('select#extension').val();
	var channel = modal.find('input[name="channel"]').val();

	if(selected_extension.length){
		selected_extension = selected_extension.split('__');
		if(selected_extension !== null && selected_extension.length === 2){
			sw.tws('redirect_call', {
				'exten': selected_extension[0],
				'context': selected_extension[1],
				'channel': channel
			})
		}else{
			toastr.warning(modal.find('input[name="invalid_data"]').val(), modal.find('h4.modal-title').text());
		}
	}else{
		toastr.warning(modal.find('input[name="invalid_data"]').val(), modal.find('h4.modal-title').text());
	}

	modal.modal('hide');
}

function sort_queued_calls(ul) {
	var members = ul.find('li.list-group-item').size();
	if(members > 1){
		ul.find('li.list-group-item').sort(function (a, b) {
			a = parseInt(a);
			b = parseInt(b);
			console.log("Sort: " + a + "," + b + ": " + (a-b));
			return a - b;
		}).appendTo(ul);
	}
}

function queued_call_notification(channel, queue) {
	if($(".WQUEUE-" + queue).length && typeof queued_calls_notifications[channel] === 'undefined'){
		var player = document.getElementById('queued-calls-notification');
		if(!queued_call_notified){
			player.play();
			queued_call_notified = true;
			queued_calls_notifications[channel] = true;
			player.onended = function () {
				if ('speechSynthesis' in window) {
					var user_lang = js_data.user_lang;
					user_lang = user_lang.split(/(.*)_(.*)/);
					var sentences = js_data.queued_calls_notifications;
					var P1 = find_local_code(user_lang[1]);
					P1.then(
						function(voice){
							var notification_lang = voice.lang;
							notification_lang = notification_lang.split(/(.*)-(.*)/);
							var queues = js_data.queues;
							var queue_name = queue;
							if(typeof queues[queue] !== 'undefined'){
								queue_name = queues[queue]['description'];
							}

							var text = 'You have a new queued call on ' +  queue_name + ' queue';
							if((typeof notification_lang[1] !== 'undefined' && typeof sentences[notification_lang[1]] !== 'undefined')){
								text = sentences[notification_lang[1]];
								text = text.replace("${QUEUE}", queue_name);
							}

							var utterance = new SpeechSynthesisUtterance(text);
							utterance.voice = voice;
							utterance.lang = voice.lang;
							utterance.onend = function(e) {
								queued_call_notified = false;
								window.speechSynthesis.cancel();
							};
							speechSynthesis.speak(utterance);
						}
					);
				}else{
					queued_call_notified = false;
				}
			};
		}
	}
}

function find_local_code(lang) {
	return new Promise(function (resolve, reject){
		for (var i=0, len=speech_voices.length; i<len; i++){
			var voice=speech_voices[i];
			if (voice.lang.indexOf(lang)===0){
				resolve(voice);
				break ;
			}
		}

		resolve(default_voice);
	});

}

function cleanQueuedCalls() {
	var widget = $(".widget-queued_calls");
	var list = widget.find('li');
	list.remove();
}
