var QMExtensions;
var pausedMembers = {};
var pausedTimers = {};

let loggedInMembers = {};
let loggedInTimers = {};

var extensions_queues_status;
var queues;

$(function () {
    addModuleCallBack('queues_widget',function () {
    	setupQueuesVariables();
	    var queue_widget = $(".widget-queues");

	    /**
	     * Function to execute when the following AMI events are sent:
	     *  - queuemember
	     *  - queuememberstatus
	     *  - queuememberadded
	     *  - queuememberpause
	     *
	     */
	    socket_io.on('queue_member', function (data) {
				//No queues on the system
				if(typeof queues === "undefined" || queues === null)
					return;

	    	//Validate if the given queue belongs to the current tenant, otherwise do nothing
		    if(queues.indexOf(data.queue) === -1)
		    	return ;

		    var collapse_list = queue_widget.find('a[href="#collapse-list-group'+ data.queue +'"]');
		    var badge = collapse_list.find('span.badge');
		    var ul = queue_widget.find('#wc-' + data.queue);
		    var member = ul.find('#member-' + data.number);
		    if(!member.length) {
			    var li = $('<li>');
			    var icon = $('<span>');
			    var description = $('<span>');
			    li.addClass('list-group-item');
			    li.attr('id', 'member-' + data.number);
			    li.attr('data-type', 'list');
			    li.attr('data-widget', ul.attr('data-widget'));

			    $.each(data, function (prop, value) {
				    li.attr('data-' + prop, value);
			    });

			    icon.addClass('fas fa-user icon');
			    description.addClass('description f-size-12');
			    description.text(getQueueMemberName(data.number));

			    icon.appendTo(li);
			    description.appendTo(li);
			    li.appendTo(ul);

			    //If member didn't exist, then the new created item to member variable is assigned
			    member = li;
		    }

		    var icon = member.find('span.icon');
		    var final_status = (data.paused === '1' && data.status === 'not_in_use' ? 'paused' : data.status);
		    member.attr('data-paused', data.paused);
		    member.attr('data-status', final_status);
		    member.attr('title', '' +
			    'Status: ' + capitalize((final_status).replace(/_/g, ' ')) +
			    '\nType: ' + capitalize(data.membership) +
			    '\nCalls Taken: ' + data.callstaken);

		    icon.removeClass(function (index, className) {
			    return (className.match(/fa-[a-z&-]+/) || []).join(' ');
		    });

		    switch (final_status){
			    case 'not_in_use':
			    case 'busy':
				    icon.addClass('fa-user');
			    	break;
			    case 'ringing':
			    case 'in_use':
			    case 'onhold':
				    icon.addClass('fa-headset');
				    break;
			    case 'invalid':
			    case 'unavailable':
				    icon.addClass('fa-user-slash');
			    	break;
			    case 'paused':
				    icon.addClass('fa-user-clock');
				    break;
		    }


		    if(!badge.length){
			    var b_span = $("<span>");
			    b_span.addClass('badge badge-success pull-right');
			    b_span.text('1');
			    b_span.appendTo(collapse_list);
		    }else{
			    badge.text(ul.find('li.list-group-item').size());
		    }

		    if(typeof extensions_queues_status[data.number] === "undefined")
		    	return ;

		    if(typeof extensions_queues_status[data.number][data.queue] !== "undefined"){
			    //Save in the array that the member is logged in and define if is paused or not
			    extensions_queues_status[data.number][data.queue]['logged_in'] = 'yes'; //If an event comes here, means that the user is logged in
			    extensions_queues_status[data.number][data.queue]['paused'] = (data.paused === '1' ? 'yes' : 'no');
			    extensions_queues_status[data.number][data.queue]['paused_reason'] = data.pausedreason;
			    extensions_queues_status[data.number][data.queue]['in_call'] = (data.status === 'in_use' ? 'yes' : 'no');
			    setQueuesStatus(extensions_queues_status);
			    ExtensionSummarySetQueuesStatus(extensions_queues_status);
		    }

		    pauseMember(data);
		    queue_members_summary(data);
		    my_queues(data);
		    extensions_info(data);
	    });

	    socket_io.on('remove_member', function (data) {
		    var collapse_list = queue_widget.find('a[href="#collapse-list-group'+ data.queue +'"]');
		    var badge = collapse_list.find('span.badge');
		    var ul = queue_widget.find('#wc-' + data.queue);
		    ul.find('#member-' + data.number).remove();
		    if(ul.find('li.list-group-item').size() <= 0){
			    badge.remove();
		    }else{
					badge.text(ul.find('li.list-group-item').size());
				}

		    if(typeof extensions_queues_status[data.number][data.queue] !== "undefined"){
			    //Save in a array that the member was logged out
			    extensions_queues_status[data.number][data.queue]['logged_in'] = 'no';
			    setQueuesStatus(extensions_queues_status);
			    ExtensionSummarySetQueuesStatus(extensions_queues_status);
		    }

		    queue_members_summary_removed(data);
		    my_queues_member_removed(data);
	    });

	    queues_context_menu();
	    member_context_menu();
	    launchModuleCallBack('queues_stats_summary_widget');
    });

    addModuleCallBack('queues_widget_settings',function () {
	    launchModuleCallBack('layouts_build_widget');
	    dynamic_queues_layout();
    });
});

function setupQueuesVariables() {
	QMExtensions = js_data.extensions_auto_complete;
	extensions_queues_status = js_data.extensions_queues_status;
	queues = js_data.tenant_queues;
}

function queues_context_menu(){
    $.contextMenu({
        selector: '#switchboard-monitor .widget-queues .wid-item-actions',
	    trigger: getContextualMenuTrigger(),
        build: function(trigger, event) {
            var options = {
                callback: function(key) {
                    var member = key.split('/');
                    socket_io.emit('queueadd', {
                        'queue': trigger.attr('data-queue'),
                        'context': member[0],
	                    'hints_context': js_data.tenant_prefix +'extension-hints',
                        'extension': member[1],
                        'name': member[2],
	                    'penalty': member[3]
                    });

                   update_queue_member_hint(member[1], trigger.attr('data-queue'), 'login', true);
                },
	            delay: 500,
	            autoHide: true,
                items: {}
            };

            var queue = trigger.attr('data-queue');
            var members = $('ul#wc-' + queue).find('li').map(function() {return ($(this).attr('id')).split('-')[1];}).get();
            $.ajax({
                type: "POST",
                dataType: "JSON",
                url: window.location.pathname, //Only made request to the actual location
                async: false,
                context: $(this),
                data: {
                    action: $('div#switchboard-monitor').find('input[name="wm_action"]').val(),
                    widget : trigger.attr('data-widget'),
                    queue : trigger.attr('data-queue-id'),
                    members : members,
                    status : 'queue'
                },
                success: function(response){
                    options.items = response;
                },
                error: process_server_error
            });

            return options;
        }

    });
}

function member_context_menu(){
    $.contextMenu({
        selector: '.widget-queues .list-group-item',
	    trigger: getContextualMenuTrigger(),
        build: function(trigger, event) {
            var options = {
                callback: function(key, opt) {
                    var remove_match = key.match(/^queue_remove_member-(.*)/);
                    if(remove_match){
                        socket_io.emit('queueremove', {
                            'queue': trigger.attr('data-queue'),
                            'context': trigger.attr('data-context'),
                            'extension': remove_match[1]
                        });

	                    update_queue_member_hint(remove_match[1], trigger.attr('data-queue'), 'login', false);
                    }

                    var pause_match = key.match(/^pause-(.*)/);
                    if(pause_match){
                        pause_unpause_member(trigger, pause_match[1], true, '');
                    }

                    var pause_wr_match = key.match(/^pwreason\d+-(.*)/);
                    if(pause_wr_match){
                        var li = opt.$selected;
                        pause_unpause_member(trigger, pause_wr_match[1], true, li.text());
                    }

                    var pause_wcr_match = key.match(/^pcustom-(.*)/);
                    if(pause_wcr_match){
	                    swal({
			                    title: 'Custom Pause',
			                    text: 'Pause Reason',
			                    confirmButtonText: 'Pause',
			                    type: "input",
			                    showCancelButton: true,
			                    closeOnConfirm: false,
			                    animation: "slide-from-top",
			                    inputPlaceholder: 'Write your pause reason'
		                    },
		                    function(inputValue){
			                    if (inputValue === false) return false;

			                    if (inputValue === "") {
				                    swal.showInputError('You did not write any pause reason');
				                    return false
			                    }

			                    pause_unpause_member(trigger, pause_wcr_match[1], true, inputValue);
			                    swal.close();
		                    });
                    }

                    var unpause_match = key.match(/^unpause-(.*)/);
                    if(unpause_match){
	                    pause_unpause_member(trigger, unpause_match[1], false, '');
                    }

                },
	            delay: 500,
	            autoHide: true,
                items: {}
            };

            $.ajax({
                type: "POST",
                dataType: "JSON",
                url: window.location.pathname, //Only made request to the actual location
                async: false,
                context: $(this),
                data: {
                    action: $('div#switchboard-monitor').find('input[name="wm_action"]').val(),
                    widget : trigger.attr('data-widget'),
                    member : (trigger.attr('id')).split('-')[1],
                    status : trigger.attr('data-membership'),
	                paused : trigger.attr('data-paused')
                },
                success: function(response){
                    options.items = response;
                },
                error: process_server_error
            });

            return options;
        }

    });
}

function capitalize(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
}

function dynamic_queues_layout() {
	var form = $("#m-form-l-widget-queues");
	var multiple_dropdown = form.find("select#items");
	multiple_dropdown.multiselect({'sortable': 'both', 'locale': js_data.user_lang});
}

/*
* Update the agents information on queues summary widget
*/
function queue_members_summary(data) {
    var widget = $(".widget-queue_members");
    var status = widget.find('.qa-' + data.number + '-' + data.queue + '.status');
    var incall = widget.find('.qa-' + data.number + '-' + data.queue + '.incall');
    var callstaken = widget.find('.qa-' + data.number + '-' + data.queue + '.callstaken');
    var lastcall = widget.find('.qa-' + data.number + '-' + data.queue + '.lastcall');
    var pausedreason = widget.find('.qa-' + data.number + '-' + data.queue + '.pausedreason');

    incall.html(data.status === 'in_use' ?  js_data.yes : js_data.no);
    callstaken.html(data.callstaken);
    if(data.lastcall !== '0'){
	    lastcall.html(moment.unix(data.lastcall).fromNow());
    }else{
	    lastcall.html('--');
    }

		//Login Information
		if(data.event === "QueueMemberAdded" || data.event === "QueueMember" || data.event === "QueueMemberPause"){
			let active_class = data.paused === '1' ? 'fa-pause-circle pause' : 'fa-circle logged_in';
			status.find('i').attr('class','fa ' + active_class);
			status.attr('data-status', 'logged_in');
			status.attr('data-paused', data.paused);

			loginMember(data)
		}

    pausedreason.html(data.pausedreason);
}

function extensions_info(data) {
	var widget = $(".widget-extensions");
	var extension = widget.find('.extension-' + data.number);
	var info =  extension.find('div.ext-info');
	var icon = info.children('i');
	var dnd = extension.hasClass('dnd-active');

	if( data.paused == '1' && !dnd){
		icon.attr('title', data.pausedreason)
	}else if (data.paused == '1' && dnd){
		icon.attr('title', 'DND' + ' + '+ data.pausedreason)
	} else{
		if(dnd){
			icon.attr('title', 'DND');
		}else{
			icon.attr('title', '');
		}
	}

}


function queue_members_summary_removed(data) {
	var widget = $(".widget-queue_members");
	var status = widget.find('.qa-' + data.number + '-' + data.queue + '.status');

	status.find('i').attr('class','fa fa-circle logged_out');
	status.attr('data-status', 'logged_out');
	status.attr('data-paused', 0);

	logoutMember(data)
}

function my_queues(data) {
	var widget = $(".widget-my_queues");
	var status = widget.find('.myq-' + data.number + '-' + data.queue + '.status');
	var callstaken = widget.find('.myq-' + data.number + '-' + data.queue + '.callstaken');

	callstaken.html(data.callstaken);
	var active_class = data.paused === '1' ? 'fa-pause-circle pause' : 'fa-circle logged_in';
	status.find('i').attr('class','fa ' + active_class);
	status.attr('data-status', 'logged_in');
	status.attr('data-paused', data.paused);

}

function my_queues_member_removed(data) {
	var widget = $(".widget-my_queues");
	var status = widget.find('.myq-' + data.number + '-' + data.queue + '.status');
	status.find('i').attr('class','fa fa-circle logged_out');
	status.attr('data-status', 'logged_out');
	status.attr('data-paused', 0);
}

function pause_unpause_member(trigger, extension, pause, reason) {
	socket_io.emit('queuepause', {
		'queue': trigger.attr('data-queue'),
		'context': trigger.attr('data-context'),
		'extension': extension,
		'paused': pause,
        'reason': reason
	});

	update_queue_member_hint(extension, trigger.attr('data-queue'), 'pause', pause);
}

function getQueueMemberName(extension){
	var name = extension;
	$.each(QMExtensions, function(key, item){
		if(item.extension == extension){
			name = item.description;
			return false; //break
		}
	});

	return name;
}

function loginMember(data){
	let member = data.number+'-'+data.queue;
	if(!loggedInMembers.hasOwnProperty(member)){
		loggedInMembers[member] = data.logintime;
	}

	let widget = $("#switchboard-monitor").find(".widget-queue_members");
	let loginTimeCell = widget.find('.qa-' + member + '.last-login');

	let lastLoginTime = parseInt(loggedInMembers[member]);
	if(!isNaN(lastLoginTime)){
		loginTimeCell.html(moment.unix(lastLoginTime).fromNow());
		loggedInTimers[member] = setTimeout(loginMember.bind(null, data), 30000);
	}else{
		logoutMember(data);
	}
}

function logoutMember(data){
	let member = data.number+'-'+data.queue;

	clearTimeout(loggedInTimers[member]);
	delete loggedInMembers[member];

	let widget = $("#switchboard-monitor").find(".widget-queue_members");
	let loginTimeCell = widget.find('.qa-' + member + '.last-login');
	loginTimeCell.html('--');
}

function pauseMember(data) {
	var paused = (data.paused == "1" ? true : false);
	var member = data.number+'-'+data.queue;
	if(paused && !pausedMembers.hasOwnProperty(member)){
		pause_member_timer(member, data.paused_time);
	}else if(!paused && pausedTimers.hasOwnProperty(member)){
		clean_member_timer(member);
	}
}

function clean_member_timer(member) {
	clearTimeout(pausedTimers[member]);
	delete pausedMembers[member];

	//Queue Members Widget
	var widget = $("#switchboard-monitor").find(".widget-queue_members");
	var pausetime = widget.find('.qa-' + member + '.pausetime');
	pausetime.html('--:--:--');

	//My Queues Widget
	widget = $("#switchboard-monitor").find(".widget-my_queues");
	pausetime = widget.find('.myq-' + member + '.pausetime');
	pausetime.html('--:--:--');

}

function pause_member_timer(member, paused_time) {
	if(!pausedMembers.hasOwnProperty(member)){
		pausedMembers[member] = paused_time;
	}else{
		if(paused_time > 0){
			pausedMembers[member] = paused_time;
		}else{
			pausedMembers[member] += 1;
		}

	}

	//Queue Members Widget
	var widget = $("#switchboard-monitor").find(".widget-queue_members");
	var pausetime = widget.find('.qa-' + member + '.pausetime');
	pausetime.html(toHHMMSS(pausedMembers[member]));

	//My Queues Widget
	widget = $("#switchboard-monitor").find(".widget-my_queues");
	pausetime = widget.find('.myq-' + member + '.pausetime');
	pausetime.html(toHHMMSS(pausedMembers[member]));

	pausedTimers[member] = setTimeout(pause_member_timer.bind(null, member, 0), 1000);
}

function cleanQueueMembers(){
	//Queues Widget
	var queue_widget = $(".widget-queues");
	var dynamicMembers = queue_widget.find('li[data-membership="dynamic"]');
	var badge = queue_widget.find('span.badge');
	badge.remove();
	dynamicMembers.remove();

	//My Queues Widget
	var myQueues = $(".widget-my_queues");
	var status = myQueues.find('.status');
	status.find('i').attr('class','fa fa-circle logged_out');
	status.attr('data-status', 'logged_out');
	status.attr('data-paused', 0);

	//Queue Members Summary
	var queueMembersSummary = $(".widget-queue_members");
	var QMstatus = queueMembersSummary.find('.status');
	QMstatus.find('i').attr('class','fa fa-circle logged_out');
	QMstatus.attr('data-status', 'logged_out');
	QMstatus.attr('data-paused', 0);
}