var trunk_widget;

$(function () {
	addModuleCallBack('trunks_widget',function () {
		setupTrunksVariables();
		socket_io.on('trunk_established_call', function (data) {
			if(data.channel.match(/^DAHDI/i)){
				telephony_call(data);
			}else{
				sip_iax_call(data);
			}
		});

		$.each(trunk_widget.find('.panel'), function () {
			var list_group = $(this).find('.list-group');
			var total_items = list_group.find('.list-group-item').length;
			var accordion_title = $(this).find('.accordion-title');
			var label = $("<span class='label label-success accordion-label'>");

			var technology = $(this).find('.collapse-list').data('technology');
			var label_text = (technology !== 'telephony' ? technology.toUpperCase() :  total_items);
			label.text(label_text);
			label.prependTo(accordion_title);
		});

		trunks_context_menu();
	});

	addModuleCallBack('trunks_widget_settings',function () {
		launchModuleCallBack('layouts_build_widget');
		dynamic_trunks_layout();
	});
});

function setupTrunksVariables() {
	trunk_widget = $(".widget-trunks");
}

function sip_iax_call(data) {
	var trunk_name = get_trunk_name(data.channel);
	var trunk = trunk_widget.find('a[href="#collapse-list-group' + trunk_name + '"]');

	if(trunk.length) {
		var badge = trunk.find('span.badge');
		var title = trunk.find('.accordion-title');
		var ul = trunk.closest('.panel').find('#wc-' + trunk_name);
		var call_item = ul.find('.call-item[data-channel="' + data.channel + '"]');
		var extension = data.callee;
		if(typeof devices[extension] !== 'undefined'){
			extension = devices[extension];
		}

		if (!call_item.length) {
			var li = $('<li class="list-group-item call-item">');
			li.attr('data-channel', data.channel);
			li.attr('data-type', 'list');
			li.attr('data-widget', ul.attr('data-widget'));
			li.attr('data-status','on-call');

			let phoneIcon = (data.calltype === 'incoming' ? 'fa-solid fa-phone-arrow-down-left' : 'fa-solid fa-phone-arrow-up-right');
			var icon = $(`<span style="margin-right: 2px; font-size: 0.8rem" class="${phoneIcon}">`);


			var description = $('<span class="description f-size-12">');
			var duration = $('<span class="pull-right duration">');

			description.text(extension + ' - ' + data.caller);
			duration.text(toHHMMSS(data.duration));

			icon.appendTo(li);
			description.appendTo(li);
			duration.appendTo(li);
			li.appendTo(ul);

			var elem = ul.find('.call-item[data-channel="' + data.channel + '"]');
			call_duration_timer(elem);
		} else {
			call_item.find('span.description').text(extension + ' - ' + data.caller);
			call_item.find('span.duration').text(toHHMMSS(data.duration));
		}

		title.find('.accordion-label').attr('data-status', 'busy');
		if(!badge.length) {
			trunk.append('<span class=\'badge badge-success pull-right\'>1</span>');
		} else {
			badge.text(ul.find('li.list-group-item').size());
		}
	}
}

function telephony_call(data) {
	if(data.port > 0){
		setDataChannelAttr(data.port, data.channel);
		return;
	}

	var trunk_name = get_trunk_name(data.channel);
	var channel = trunk_widget.find('.list-group-item#wi-'+trunk_name);
	var channelByChannel = trunk_widget.find('.list-group-item[data-channel="'+ data.channel +'"]');
	if(channelByChannel.length){
		channel = channelByChannel;
	}

	if(channel.length) {
		channel.attr('data-channel', data.channel);
		channel.attr('data-status','on-call');
		var extension = data.callee;
		if(typeof devices[extension] !== 'undefined'){
			extension = devices[extension];
		}

		var current_call = channel.find('span.current-call');
		if(!current_call.length) {
			current_call = $('<span class="current-call f-size-12">');
			var duration = $('<span class="pull-right duration">');
			var description = channel.find('span.description');
			var icon = channel.find('span.icon');

			if(!icon.length) {
				icon = $('<span class="fa fa-phone icon">');
				icon.prependTo(description);
			}

			current_call.text(' => ' + extension + ' - ' + data.caller);
			duration.text(toHHMMSS(data.duration));
			call_duration_timer(current_call);

			current_call.appendTo(channel);
			duration.appendTo(current_call);
		}

		var panel = channel.closest('.panel');
		var collapse_list = panel.find('a[data-toggle="collapse"]');
		var list_group = panel.find('ul.list-group');
		var badge = collapse_list.find('span.badge');
		var title = collapse_list.find('.accordion-title');
		var active_calls = list_group.find('li.list-group-item .current-call').size();

		title.find('.accordion-label').attr('data-status', 'busy');
		if(!badge.length) {
			collapse_list.append('<span class=\'badge badge-success pull-right\'>1</span>');
		} else {
			badge.text(active_calls);
		}
	}
}

function setDataChannelAttr(port, chn){
	var channel = trunk_widget.find('.list-group-item#wi-' + port);
	if(channel.length){
		channel.attr('data-channel', chn);
	}
}

function sip_iax_end_call(data) {
	var trunk_name = get_trunk_name(data.channel);
	var trunk_call_item = trunk_widget.find('.call-item[data-channel="' + data.channel + '"]');
	trunk_call_item.remove();

	var trunk = trunk_widget.find('a[href="#collapse-list-group'+ trunk_name +'"]');
	if(trunk.length) {
		var ul = trunk.closest('.panel').find('#wc-' + trunk_name);
		var badge = trunk.find('span.badge');
		var title = trunk.find('.accordion-title');
		var total_calls = ul.find('li.list-group-item').size();
		if(total_calls > 0) {
			badge.text(total_calls);
		} else {
			title.find('.accordion-label').removeAttr('data-status');
			badge.remove();
		}
	}
}

function telephony_end_call(data) {
	var channel = trunk_widget.find('.list-group-item[data-channel="'+ data.channel +'"]');
	if(channel.length) {
		var current_call = channel.find('.current-call');
		var icon = channel.find('.icon');
		channel.attr('data-status','idle');
		current_call.remove();

		var panel = channel.closest('.panel');
		var collapse_list = panel.find('a[data-toggle="collapse"]');
		var list_group = panel.find('ul.list-group');
		var badge = collapse_list.find('span.badge');
		var title = collapse_list.find('.accordion-title');
		var total_calls = list_group.find('li.list-group-item .current-call').size();
		icon.remove();
		if(total_calls > 0) {
			badge.text(total_calls);
		} else {
			title.find('.accordion-label').removeAttr('data-status');
			badge.remove();
		}
	}
}

function call_duration_timer(elem) {
	var call_interval = setInterval(function () {
		if(elem.length){
			var duration_span = elem.find('span.duration');
			var new_duration = parseInt(toSeconds(duration_span.text())) + 1;
			duration_span.text(toHHMMSS(new_duration));
		}else{
			clearInterval(call_interval);
		}
	}, 1000);
}

function get_trunk_name(channel) {
	if(typeof channel === 'undefined' || !channel.length)
		return '';

	if(channel.match(/^\d+/)){
		return channel;
	}

	var chan = channel.match(/\/(.*)\-/);
	if(chan === null)
		return '';

	chan = chan.pop();
	return chan.replace(/[@/]/g,'-');
}

function dynamic_trunks_layout() {
	var form = $("#m-form-l-widget-trunks");
	var multiple_dropdown = form.find("select#items");
	multiple_dropdown.multiselect({'sortable': 'both', 'locale': js_data.user_lang});
}

function trunks_context_menu() {
	$.contextMenu({
		selector: '#switchboard-monitor .widget-trunks li.list-group-item',
		trigger: getContextualMenuTrigger(),
		build: function (trigger, event) {
			var options = {
				callback: function (key, options) {
					if(key.match(/^hangup$/i)){
						hangup(trigger.attr('data-channel'));
					}
				},
				delay: 500,
				autoHide: true,
				items: {}
			};

			$.ajax({
				type: "POST",
				dataType: "JSON",
				url: window.location.pathname, //Only made request to the actual location
				async: false,
				context: $(this),
				data:{
					action: $('div#switchboard-monitor').find('input[name="wm_action"]').val(),
					widget : trigger.parent('ul.list-group').attr('data-widget'),
					status : trigger.attr('data-status')
				},
				success: function(response){
					options.items = response;
				},
				error: process_server_error
			});

			return options;
		}
	});
}

function trunksClean(){
	let trunk_widget = $(".widget-trunks");
	let trunk_call_item = trunk_widget.find('.call-item');

	trunk_call_item.remove();

	var badge = trunk_widget.find('span.badge');
	var title = trunk_widget.find('.accordion-title');

	title.find('.accordion-label').removeAttr('data-status');
	badge.remove();
}
