var user_extension;
var user_context;
var queues_context;
var devices;
var layout = [];
var grid = false;
let timerIsRunning = false;

$(function () {
  addModuleCallBack('switchboard',function () {
	  launchModuleCallBack('no_menu');

		//Load Data
		if(typeof socket_io!== "undefined" && socket_io.connected){
			socket_io.emit('sw_opened');
		}

	  setupSwitchboardVariables();
	  var grid_panel = $('#switchboard-monitor').find('.grid-stack-panel');

	  grid_panel.gridstack({
		  cellHeight: 80,
		  verticalMargin: 10,
		  handle: '.grid-stack-header'
	  });

	  grid = grid_panel.data('gridstack');

	   $('.minimize-widget').on('click', function () {
		  var grid_item = $(this).closest('.grid-stack-item');
		  var cur_height = grid_item.attr('data-gs-height');
		  var def_height = grid_item.attr('data-gs-def-height');
		  var minimize = (grid_item.attr('data-gs-height') > '1');

		  grid_item.attr('data-gs-def-height', (minimize ? cur_height : def_height));
		  grid.resize(grid_item, null, (minimize ? 1 : parseInt(def_height)));
		  grid.resizable(grid_item, !minimize);

		  if(minimize) {
			  $(this).removeClass('fa-window-minimize').addClass('fa-window-maximize');
		  } else {
			  $(this).addClass('fa-window-minimize').removeClass('fa-window-maximize');
		  }
	  });

	  $('.fullscreen-widget').on('click', function () {
		  var grid_item = $(this).closest('.grid-stack-item');
		  var target = grid_item[0];

		  if (!document.fullscreenElement) {
			  // If the document is not in full screen mode
			  // make the element full screen
			  target.requestFullscreen();
			  save_current_layout();
			  grid.update(grid_item, 0, 0, 12, 12);
			  $(this).addClass('on');
			  toggle_actions(true);
		  } else {
			  // Otherwise exit the full screen
			  if (document.exitFullscreen) {
				  document.exitFullscreen();
			  }

			  //Fix layout
			  toggle_actions(false);
			  load_current_layout();
		  }
	  });

	  document.addEventListener("fullscreenchange", onFullScreenChange, false);
	  document.addEventListener("webkitfullscreenchange", onFullScreenChange, false);
	  document.addEventListener("mozfullscreenchange", onFullScreenChange, false);

		//Setup Moment Locale
		let momentLocale = getMomentLocale();
		moment.locale(momentLocale);

	  loadWidgetCallBacks();
	  search_interaction();
	  extension_calls_history();
	  getCallsHistoryData();
		dataRefresher();
  });
});

function dataRefresher() {
	if (!timerIsRunning) {
		// Set the interval to 30 minutes (30 * 60 * 1000 milliseconds)
		const intervalId = setInterval(function () {
			if(typeof socket_io!== "undefined" && socket_io.connected){
				console.log("Refreshing dashboard....")
				socket_io.emit('sw_opened');
			}
		}, 30 * 60 * 1000);

		// Set the flag to indicate that the timer is running
		timerIsRunning = true;

		// Log the intervalId for future reference (optional)
		console.log("Timer started with intervalId: ", intervalId);
	} else {
		console.log("Timer is already running!");
	}
}

function onFullScreenChange(){
	if (!document.fullscreenElement) {
		console.log('Leaving full-screen mode.');
		//Fix layout
		toggle_actions(false);
		load_current_layout();
	}
}

function setupSwitchboardVariables() {
	user_extension = js_data.user_extension;
	user_context = js_data.user_context;
	queues_context = js_data.queues_context;
	devices = js_data.extensions_devices;
}

function loadWidgetCallBacks() {
	$.contextMenu( 'destroy' );

	cleanUpListeners();

	launchModuleCallBack('conferences_widget');
	launchModuleCallBack('extensions_widget');
	launchModuleCallBack('queues_widget');
	launchModuleCallBack('queue_members_widget');
	launchModuleCallBack('my_extension_widget');
	launchModuleCallBack('my_queues_widget');
	launchModuleCallBack('queued_calls_widget');
	launchModuleCallBack('queues_calls_counter_widget');
	launchModuleCallBack('queues_stats_summary_widget');
	launchModuleCallBack('trunks_widget');
	launchModuleCallBack('parking_lots_widget');
	launchModuleCallBack('extensions_summary_widget');
}

function save_current_layout() {
	$('.grid-stack').children('.grid-stack-item').each(function (key) {
		layout[key] = {
			x:      $(this).attr('data-gs-x'),
			y: 	    $(this).attr('data-gs-y'),
			width:	$(this).attr('data-gs-width'),
			height: $(this).attr('data-gs-height')
		};
	});
}

function load_current_layout() {
	var current_fullscreen_widget = $('.fullscreen-widget.on');
	$('.grid-stack').children('.grid-stack-item').each(function (key) {
		grid.update($(this), parseInt(layout[key].x), parseInt(layout[key].y), parseInt(layout[key].width), parseInt(layout[key].height));
	});
}

function toggle_actions(is_full_screen) {
	var full_screen_widget = $('.fullscreen-widget.on');
	var grid_item = full_screen_widget.closest('.grid-stack-item');
	var action_buttons = full_screen_widget.parent('.tools');
	action_buttons.find('.icon:not(.fullscreen-widget)').toggleClass('hidden');

	if(!is_full_screen)
		full_screen_widget.removeClass('on');

	grid.resizable(grid_item, !is_full_screen);
	grid.movable(grid_item, !is_full_screen);
}

function update_queue_member_hint(extension, queue, type, status) {
	if(typeof queue !== 'undefined'){
		var queues = js_data.queues;
		if(typeof queues[queue] !== 'undefined'){
			queue = queues[queue]['number'];
		}
	}

	var hints = (type === 'login' ? js_data.queue_hints.login_hints : js_data.queue_hints.pause_hints);
	var device = (typeof queue === 'undefined' ? hints.all : hints.single);
	device = device.replace(/EXTENSION|QUEUE/g, function get_device_value(x) {
		return (x === 'EXTENSION' ? extension : queue);
	});
	var state = (status === true ? 'BUSY' : 'NOT_INUSE');

	sw.tws('change_devstate', {
		'device': 'Custom:' + js_data.tenant_prefix + device,
		'state': state
	})

}

function search_interaction() {
	var search_btn = $("i.search-on-widget");
	var close_search_btn = $("a.close-widget-search");
	search_btn.unbind('click');
	search_btn.on('click', function () {
		var search = $(this).parents("div.grid-stack-header").find('div.search-field');
		search.show();
		search.find('input').focus();
	});

	close_search_btn.unbind('click');
	close_search_btn.on('click', function () {
		var search = $(this).parent('div.search-field');
		search.hide();
		search.find('input').val('').focusout();
	});
}

function extension_calls_history() {
	$(".calls-history").on('click', function () {
		var extension = $(this).attr('data-extension');
		var action = $("div#switchboard-monitor").find('input[name="c_history"]').val();
		virtual_submit({
			'action': action,
			'extension': extension
		});
	});
}

function getCallsHistoryData() {
	addModuleCallBack('dt_c_history', function () {
		var form = $("#m-form-calls-history");
		var table = form.find("#tbl-extension-calls-history");

		if(!table.length)
			return ;

		table.DataTable({
			processing: true,
			serverSide: true,
			lengthMenu: [10, 25, 50, 100, 250],
			iDisplayLength: 10,
			drawCallback: function( settings ){

			},
			ajax: {
				url: window.location.pathname,
				type: 'POST',
				data: function (d) {
					var form_data = form.serializeObject();
					form_data['dtables'] = d;
					return form_data;
				}
			},
			responsive: true,
			searching: true,
			autoWidth: false,
			bSort: false,
			columns: [
				{ "data": "date", "className": "text-left"},
				{ "data": "source", "className": "text-center"},
				{ "data": "destination", "className": "text-center"},
				{ "data": "call_type", "className": "text-center"},
				{ "data": "duration", "className": "text-center"},
				{ "data": "disposition", "className": "text-center"}
			],
			dom: 'Bfrtip',
			buttons: []
		});

		/*$("#tbl-extension-calls-history").DataTable({
			processing: true,
			serverSide: false,
			lengthMenu: [10, 25, 50, 100, 250],
			iDisplayLength: 10,
			searching: true,
			autoWidth: false,
			bSort: false,
			responsive: true
		});*/
	});
}

function cleanUpListeners(){
	//Conferences
	socket_io.removeAllListeners('conference_member');
	socket_io.removeAllListeners('conf_member_leave');
	socket_io.removeAllListeners('conference_member_muted');
	socket_io.removeAllListeners('conference_member_unmuted');

	//Extensions
	socket_io.removeAllListeners('end_call');
	socket_io.removeAllListeners('device_state');
	socket_io.removeAllListeners('call_in_process');
	socket_io.removeAllListeners('established_call');
	socket_io.removeAllListeners('sendmessagestatus');
	socket_io.removeAllListeners('extension_status');
	socket_io.removeAllListeners('parked_call_retrieved');
	socket_io.removeAllListeners('hold');
	socket_io.removeAllListeners('extension_summary');

	//Queues
	socket_io.removeAllListeners('queue_member');
	socket_io.removeAllListeners('remove_member');
	socket_io.removeAllListeners('queue_caller_join');
	socket_io.removeAllListeners('queue_caller_leave');
	socket_io.removeAllListeners('queue_params');
	socket_io.removeAllListeners('queue_times');

	//Trunks
	socket_io.removeAllListeners('trunk_established_call');

	//Parking
	socket_io.removeAllListeners('parked_call');
	socket_io.removeAllListeners('parked_call_hangup');
}